package org.sonatype.tycho.p2.eclipse.bridge.impl;

import java.io.File;

import org.codehaus.plexus.archiver.UnArchiver;
import org.codehaus.plexus.logging.Logger;
import org.codehaus.plexus.util.FileUtils;
import org.sonatype.tycho.p2.eclipse.bridge.EclipseLocation;

public class PackedEclipseLocation
    implements EclipseLocation
{

    private final UnArchiver unArchiver;

    private final File eclipseArchive;

    private File location;

    private final Logger logger;

    private boolean extracted;

    private final boolean reuseExisting;

    public PackedEclipseLocation( final File eclipseArchive, final UnArchiver unArchiver, final Logger logger )
    {
        this.unArchiver = unArchiver;
        this.logger = logger;
        this.eclipseArchive = eclipseArchive;
        this.reuseExisting = false;

        try
        {
            final File tempFile = File.createTempFile( "eclipse", "" );
            tempFile.delete();
            tempFile.mkdirs();
            tempFile.deleteOnExit();
            location = new File( tempFile.getAbsolutePath(), "eclipse" );
        }
        catch ( final Exception e )
        {
            throw new RuntimeException( "Cannot unpack Eclipse", e );
        }
    }

    public PackedEclipseLocation( final File eclipseArchive, final File location, final boolean reuseExisting,
                                  final UnArchiver unArchiver, final Logger logger )
    {
        this.location = new File( location, "eclipse" );
        this.reuseExisting = reuseExisting;
        this.unArchiver = unArchiver;
        this.logger = logger;
        this.eclipseArchive = eclipseArchive;
    }

    public File get()
    {
        if ( !extracted )
        {
            try
            {
                if ( !reuseExisting )
                {
                    if ( location.exists() )
                    {
                        FileUtils.deleteDirectory( location );
                        if ( location.exists() )
                        {
                            throw new RuntimeException( String.format( "Cannot delete existing eclipse location %s", location.getAbsolutePath() ) );
                        }
                    }

                    location.mkdirs();
                    unArchiver.setSourceFile( eclipseArchive );
                    unArchiver.setDestDirectory( location.getParentFile() );
                    unArchiver.extract();

                }
                extracted = true;

                logger.debug( String.format( "Using unpacked eclipse from %s", location.getAbsolutePath() ) );
            }
            catch ( final Exception e )
            {
                throw new RuntimeException( "Cannot unpack Eclipse", e );
            }
        }
        return location;
    }
}
